import { CookieJar } from 'tough-cookie';

type Fetch = typeof globalThis.fetch;
type Credential = {
    username: string;
    password: string;
};
type CredentialProvider = () => Credential | Promise<Credential>;
type HelperConfig = {
    provider?: CredentialProvider;
    fetch?: Fetch;
    cookieJar?: CookieJar;
    generatePreviewUrlForFirstPage?: boolean;
};
declare enum FailReason {
    NO_CREDENTIAL = "no credential provided",
    ERROR_FETCH_FROM_ID = "could not fetch ticket from id.tsinghua.edu.cn",
    BAD_CREDENTIAL = "bad credential",
    ERROR_ROAMING = "could not roam to learn.tsinghua.edu.cn",
    NOT_LOGGED_IN = "not logged in or login timeout",
    NOT_IMPLEMENTED = "not implemented",
    INVALID_RESPONSE = "invalid response",
    UNEXPECTED_STATUS = "unexpected status"
}
interface ApiError {
    reason: FailReason;
    extra?: unknown;
}
declare enum SemesterType {
    FALL = "fall",
    SPRING = "spring",
    SUMMER = "summer",
    UNKNOWN = ""
}
declare enum ContentType {
    NOTIFICATION = "notification",
    FILE = "file",
    HOMEWORK = "homework",
    DISCUSSION = "discussion",
    QUESTION = "question"
}
interface IUserInfo {
    name: string;
    department: string;
}
type UserInfo = IUserInfo;
interface ISemesterInfo {
    id: string;
    startDate: Date;
    endDate: Date;
    startYear: number;
    endYear: number;
    type: SemesterType;
}
type SemesterInfo = ISemesterInfo;
declare enum CourseType {
    STUDENT = "student",
    TEACHER = "teacher"
}
interface ICourseInfo {
    id: string;
    name: string;
    chineseName: string;
    englishName: string;
    timeAndLocation: string[];
    url: string;
    teacherName: string;
    teacherNumber: string;
    courseNumber: string;
    courseIndex: number;
    courseType: CourseType;
}
type CourseInfo = ICourseInfo;
interface IRemoteFile {
    id: string;
    name: string;
    downloadUrl: string;
    previewUrl: string;
    size: string;
}
type RemoteFile = IRemoteFile;
interface INotification {
    id: string;
    title: string;
    content: string;
    hasRead: boolean;
    url: string;
    markedImportant: boolean;
    publishTime: Date;
    publisher: string;
}
interface INotificationDetail {
    attachment?: RemoteFile;
}
type Notification = INotification & INotificationDetail;
interface IFile {
    id: string;
    /** size in byte */
    rawSize: number;
    /** inaccurate size description (like '1M') */
    size: string;
    title: string;
    description: string;
    uploadTime: Date;
    /** for teachers, this url will not initiate download directly */
    downloadUrl: string;
    /** preview is not supported on all types of files, check before use */
    previewUrl: string;
    isNew: boolean;
    markedImportant: boolean;
    visitCount: number;
    downloadCount: number;
    fileType: string;
    /** for compatibility */
    remoteFile: RemoteFile;
}
type File = IFile;
interface IHomeworkStatus {
    submitted: boolean;
    graded: boolean;
}
declare enum HomeworkGradeLevel {
    /** 已阅 */
    CHECKED = "checked",
    A_PLUS = "A+",
    A = "A",
    A_MINUS = "A-",
    B_PLUS = "B+",
    /** 优秀 */
    DISTINCTION = "distinction",
    B = "B",
    B_MINUS = "B-",
    C_PLUS = "C+",
    C = "C",
    C_MINUS = "C-",
    G = "G",
    D_PLUS = "D+",
    D = "D",
    /** 免课 */
    EXEMPTED_COURSE = "exempted course",
    P = "P",
    EX = "EX",
    /** 免修 */
    EXEMPTION = "exemption",
    /** 通过 */
    PASS = "pass",
    /** 不通过 */
    FAILURE = "failure",
    W = "W",
    I = "I",
    /** 缓考 */
    INCOMPLETE = "incomplete",
    NA = "NA",
    F = "F"
}
interface IHomework extends IHomeworkStatus {
    id: string;
    studentHomeworkId: string;
    title: string;
    deadline: Date;
    url: string;
    submitUrl: string;
    submitTime?: Date;
    grade?: number;
    /** some homework has levels but not grades, like A/B/.../F */
    gradeLevel?: HomeworkGradeLevel;
    gradeTime?: Date;
    graderName?: string;
    gradeContent?: string;
}
interface IHomeworkDetail {
    description?: string;
    /** attachment from teacher */
    attachment?: RemoteFile;
    /** answer from teacher */
    answerContent?: string;
    answerAttachment?: RemoteFile;
    /** submitted content from student */
    submittedContent?: string;
    submittedAttachment?: RemoteFile;
    /** grade from teacher */
    gradeAttachment?: RemoteFile;
}
type Homework = IHomework & IHomeworkDetail;
declare enum HomeworkCompletionType {
    INDIVIDUA = 1,
    GRUOP = 2
}
declare enum HomeworkSubmissionType {
    WEB_LEARNING = 2,
    OFFLINE = 0
}
interface IHomeworkTA {
    id: string;
    index: number;
    title: string;
    description: string;
    publisherId: string;
    publishTime: Date;
    startTime: Date;
    deadline: Date;
    url: string;
    completionType: HomeworkCompletionType;
    submissionType: HomeworkSubmissionType;
    gradedCount: number;
    submittedCount: number;
    unsubmittedCount: number;
}
type HomeworkTA = IHomeworkTA;
interface IHomeworkSubmitAttachment {
    filename: string;
    content: Blob;
}
interface IHomeworkSubmitResult {
    result: 'success' | 'error';
    msg: string;
    object: unknown;
}
interface IDiscussionBase {
    id: string;
    title: string;
    publisherName: string;
    publishTime: Date;
    lastReplierName: string;
    lastReplyTime: Date;
    visitCount: number;
    replyCount: number;
}
interface IDiscussion extends IDiscussionBase {
    url: string;
    boardId: string;
}
type Discussion = IDiscussion;
interface IQuestion extends IDiscussionBase {
    url: string;
    question: string;
}
type Question = IQuestion;
type Content = Notification | File | Homework | Discussion | Question;
interface ICourseContent {
    [id: string]: Content[];
}
type CourseContent = ICourseContent;
interface CalendarEvent {
    location: string;
    status: string;
    startTime: string;
    endTime: string;
    date: string;
    courseName: string;
}
declare enum Language {
    ZH = "zh",
    EN = "en"
}

/** add CSRF token to any request URL as parameters */
declare const addCSRFTokenToUrl: (url: string, token: string) => string;
/** the main helper class */
declare class Learn2018Helper {
    #private;
    previewFirstPage: boolean;
    /** you can provide a CookieJar and / or CredentialProvider in the configuration */
    constructor(config?: HelperConfig);
    /** fetch CSRF token from helper (invalid after login / re-login), might be '' if not logged in */
    getCSRFToken(): string;
    /** login is necessary if you do not provide a `CredentialProvider` */
    login(username?: string, password?: string): Promise<void>;
    /**  logout (to make everyone happy) */
    logout(): Promise<void>;
    /** get user's name and department */
    getUserInfo(courseType?: CourseType): Promise<UserInfo>;
    /**
     * Get calendar items during the specified period (in yyyymmdd format).
     * @param startDate start date (inclusive)
     * @param endDate end date (inclusive)
     * If the API returns any error, this function will throw `FailReason.INVALID_RESPONSE`,
     * and we currently observe a limit of no more that 29 days.
     * Otherwise it will return the parsed data (might be empty if the period is too far away from now)
     */
    getCalendar(startDate: string, endDate: string, graduate?: boolean): Promise<CalendarEvent[]>;
    getSemesterIdList(): Promise<string[]>;
    getCurrentSemester(): Promise<SemesterInfo>;
    /** get all courses in the specified semester */
    getCourseList(semesterID: string, courseType?: CourseType): Promise<CourseInfo[]>;
    /**
     * Get certain type of content of all specified courses.
     * It actually wraps around other `getXXX` functions. You can ignore the failure caused by certain courses.
     */
    getAllContents(courseIDs: string[], type: ContentType, courseType?: CourseType, allowFailure?: boolean): Promise<CourseContent>;
    /** Get all notifications （课程公告） of the specified course. */
    getNotificationList(courseID: string, courseType?: CourseType): Promise<Notification[]>;
    /** Get all files （课程文件） of the specified course. */
    getFileList(courseID: string, courseType?: CourseType): Promise<File[]>;
    /** Get all homeworks （课程作业） of the specified course. */
    getHomeworkList(courseID: string): Promise<Homework[]>;
    getHomeworkList(courseID: string, courseType: CourseType.STUDENT): Promise<Homework[]>;
    getHomeworkList(courseID: string, courseType: CourseType.TEACHER): Promise<HomeworkTA[]>;
    /** Get all discussions （课程讨论） of the specified course. */
    getDiscussionList(courseID: string, courseType?: CourseType): Promise<Discussion[]>;
    /**
     * Get all notifications （课程答疑） of the specified course.
     * The student version supports only answered questions, while the teacher version supports all questions.
     */
    getAnsweredQuestionList(courseID: string, courseType?: CourseType): Promise<Question[]>;
    private getHomeworkListAtUrl;
    private parseNotificationDetail;
    private parseHomeworkDetail;
    private parseHomeworkFile;
    private parseDiscussionBase;
    submitHomework(studentHomeworkID: string, content?: string, attachment?: IHomeworkSubmitAttachment, removeAttachment?: boolean): Promise<IHomeworkSubmitResult>;
    setLanguage(lang: Language): Promise<void>;
    getCurrentLanguage(): Language;
}

export { ApiError, CalendarEvent, Content, ContentType, CourseContent, CourseInfo, CourseType, Credential, CredentialProvider, Discussion, FailReason, Fetch, File, HelperConfig, Homework, HomeworkCompletionType, HomeworkGradeLevel, HomeworkSubmissionType, HomeworkTA, IDiscussionBase, IHomework, IHomeworkDetail, IHomeworkStatus, IHomeworkSubmitAttachment, IHomeworkSubmitResult, IHomeworkTA, INotification, INotificationDetail, Language, Learn2018Helper, Notification, Question, RemoteFile, SemesterInfo, SemesterType, UserInfo, addCSRFTokenToUrl };
